/*
	jQuery Colorbox language configuration
	language: Brazilian Portuguese (pt-BR)
	translated by: ReinaldoMT
*/
jQuery.extend(jQuery.colorbox.settings, {
	current: "Imagem {current} de {total}",
	previous: "Anterior",
	next: "Próxima",
	close: "Fechar",
	slideshowStart: "iniciar apresentação de slides",
	slideshowStop: "parar apresentação de slides",
	xhrError: "Erro ao carregar o conteúdo.",
	imgError: "Erro ao carregar a imagem."
});