# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "",
    "(active)": "",
    "Also available in:": "",
    "Archive": "",
    "Authors": "",
    "Categories": "",
    "Comments": "",
    "LANGUAGE": "",
    "Languages:": "",
    "More posts about %s": "",
    "Newer posts": "",
    "Next post": "",
    "No posts found.": "",
    "Nothing found.": "",
    "Older posts": "",
    "Original site": "",
    "Posted:": "",
    "Posts about %s": "",
    "Posts by %s": "",
    "Posts for year %s": "",
    "Posts for {month} {day}, {year}": "",
    "Posts for {month} {year}": "",
    "Previous post": "",
    "Publication date": "",
    "RSS feed": "",
    "Read in English": "",
    "Read more": "",
    "Skip to main content": "",
    "Source": "",
    "Subcategories:": "",
    "Tags and Categories": "",
    "Tags": "",
    "Uncategorized": "",
    "Updates": "",
    "Write your page here.": "",
    "Write your post here.": "",
    "old posts, page %d": "",
    "page %d": "",
}
