/*
	jQuery Colorbox language configuration
	language: Finnish (fi)
	translated by: Mikko
*/
jQuery.extend(jQuery.colorbox.settings, {
	current: "Kuva {current} / {total}",
	previous: "Edellinen",
	next: "Seuraava",
	close: "Sulje",
	xhrError: "Sisällön lataaminen epäonnistui.",
	imgError: "Kuvan lataaminen epäonnistui.",
	slideshowStart: "Aloita kuvaesitys.",
	slideshowStop: "Lopeta kuvaesitys."	
});
