# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "%d минут чтения осталось",
    "(active)": "(активная)",
    "Also available in:": "Также доступно на:",
    "Archive": "Архив",
    "Authors": "Разработчики",
    "Categories": "Категории",
    "Comments": "Комментарии",
    "LANGUAGE": "Русский",
    "Languages:": "Языки:",
    "More posts about %s": "Больше записей о %s",
    "Newer posts": "Новые записи",
    "Next post": "Следующая запись",
    "No posts found.": "Записей не найдено.",
    "Nothing found.": "Ничего не найдено.",
    "Older posts": "Старые записи",
    "Original site": "Оригинальный сайт",
    "Posted:": "Опубликовано:",
    "Posts about %s": "Записи о %s",
    "Posts by %s": "Запись %s",
    "Posts for year %s": "Записи за %s год",
    "Posts for {month} {day}, {year}": "Записи за {day} {month} {year}",
    "Posts for {month} {year}": "Записи за {month} {year}",
    "Previous post": "Предыдущая запись",
    "Publication date": "Дата опубликования",
    "RSS feed": "RSS лента",
    "Read in English": "Прочесть по-русски",
    "Read more": "Читать далее",
    "Skip to main content": "Перейти к главному содержимому",
    "Source": "Источник",
    "Subcategories:": "Подкатегории:",
    "Tags and Categories": "Тэги и категории",
    "Tags": "Тэги",
    "Uncategorized": "Несортированное",
    "Updates": "Обновления",
    "Write your page here.": "Создайте Вашу страницу здесь.",
    "Write your post here.": "Создайте Вашу запись здесь.",
    "old posts, page %d": "%d страница со старыми записями",
    "page %d": "%d страница",
}
