# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "še %d min za branje preostanka",
    "(active)": "",
    "Also available in:": "Na voljo tudi v:",
    "Archive": "Arhiv",
    "Authors": "",
    "Categories": "Kategorije",
    "Comments": "Komentarji",
    "LANGUAGE": "Slovenščina",
    "Languages:": "Jeziki:",
    "More posts about %s": "Več objav o %s",
    "Newer posts": "Novejše objave",
    "Next post": "Naslednja objava",
    "No posts found.": "Ni najdenih objav.",
    "Nothing found.": "Brez zadetkov.",
    "Older posts": "Starejše objave",
    "Original site": "Izvorna spletna stran",
    "Posted:": "Objavljeno:",
    "Posts about %s": "Objave o %s",
    "Posts by %s": "",
    "Posts for year %s": "Objave za leto %s",
    "Posts for {month} {day}, {year}": "Objave za {day}. {month}, {year}",
    "Posts for {month} {year}": "Objave za {month} {year}",
    "Previous post": "Prejšnja objava",
    "Publication date": "Datum objave",
    "RSS feed": "vir RSS",
    "Read in English": "Beri v slovenščini",
    "Read more": "Več o tem",
    "Skip to main content": "Preskoči na glavno vsebino",
    "Source": "Izvor",
    "Subcategories:": "",
    "Tags and Categories": "Značke in kategorije",
    "Tags": "Značke",
    "Uncategorized": "",
    "Updates": "",
    "Write your page here.": "",
    "Write your post here.": "",
    "old posts, page %d": "stare objave, stran %d",
    "page %d": "stran %d",
}
