# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "%d minuta preostalo za čitanje",
    "(active)": "(aktivno)",
    "Also available in:": "Također dostupno i u:",
    "Archive": "Arhiva",
    "Authors": "",
    "Categories": "Kategorije",
    "Comments": "Komentari",
    "LANGUAGE": "hrvatski",
    "Languages:": "Jezici:",
    "More posts about %s": "Više postova o %s",
    "Newer posts": "Noviji postovi",
    "Next post": "Sljedeći post",
    "No posts found.": "Nema postova.",
    "Nothing found.": "Nema ničeg.",
    "Older posts": "Stariji postovi",
    "Original site": "Izvorna stranica",
    "Posted:": "Objavljeno:",
    "Posts about %s": "Postovi o %s",
    "Posts by %s": "",
    "Posts for year %s": "Postovi za godinu %s",
    "Posts for {month} {day}, {year}": "",
    "Posts for {month} {year}": "Postovi za {month} {year}",
    "Previous post": "Prethodni post",
    "Publication date": "Nadnevak objave",
    "RSS feed": "RSS kanal",
    "Read in English": "Čitaj na hrvatskom",
    "Read more": "Čitaj dalje",
    "Skip to main content": "Preskoči na glavni sadržaj",
    "Source": "Izvor",
    "Subcategories:": "Podkategorije:",
    "Tags and Categories": "Tagovi i kategorije",
    "Tags": "Tagovi",
    "Uncategorized": "",
    "Updates": "",
    "Write your page here.": "",
    "Write your post here.": "",
    "old posts, page %d": "stari postovi, stranice %d",
    "page %d": "stranice %d",
}
