# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "quedan %d minutos de lectura",
    "(active)": "(activo)",
    "Also available in:": "También disponible en:",
    "Archive": "Archivo",
    "Authors": "Autores",
    "Categories": "Categorías",
    "Comments": "Comentarios",
    "LANGUAGE": "Español",
    "Languages:": "Idiomas:",
    "More posts about %s": "Más publicaciones sobre %s",
    "Newer posts": "Publicaciones posteriores",
    "Next post": "Siguiente publicación",
    "No posts found.": "No se encontraron publicaciones.",
    "Nothing found.": "No se encontró nada.",
    "Older posts": "Publicaciones anteriores",
    "Original site": "Sitio original",
    "Posted:": "Publicado:",
    "Posts about %s": "Publicaciones sobre %s",
    "Posts by %s": "Publicaciones de %s",
    "Posts for year %s": "Publicaciones del año %s",
    "Posts for {month} {day}, {year}": "Publicaciones del {day} de {month} de {year}",
    "Posts for {month} {year}": "Posts de {month} de {year}",
    "Previous post": "Publicación anterior",
    "Publication date": "Fecha de publicación",
    "RSS feed": "Canal RSS",
    "Read in English": "Leer en español",
    "Read more": "Leer más",
    "Skip to main content": "Ir al contenido principal",
    "Source": "Código fuente",
    "Subcategories:": "Subcategorías:",
    "Tags and Categories": "Etiquetas y Categorías",
    "Tags": "Etiquetas",
    "Uncategorized": "Sin categoría",
    "Updates": "Actualizaciones",
    "Write your page here.": "Escriba su página aquí.",
    "Write your post here.": "Escriba su publicación aquí.",
    "old posts, page %d": "publicaciones antiguas, página %d",
    "page %d": "página %d",
}
