# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "%d min restantes para ler",
    "(active)": "(activo)",
    "Also available in:": "Tamén dispoñible en:",
    "Archive": "Archivo",
    "Authors": "Autores",
    "Categories": "Categorías",
    "Comments": "Comentarios",
    "LANGUAGE": "Inglés",
    "Languages:": "Linguas:",
    "More posts about %s": "Máis artigos sobre %s",
    "Newer posts": "Últimos artigos",
    "Next post": "Seguinte artigo",
    "No posts found.": "Non se atoparon artigos.",
    "Nothing found.": "Non se atopou nada.",
    "Older posts": "Artigos vellos",
    "Original site": "Sitio orixinal",
    "Posted:": "Publicado:",
    "Posts about %s": "Artigos sobre %s",
    "Posts by %s": "Publicacións de %s",
    "Posts for year %s": "Artigos do ano %s",
    "Posts for {month} {day}, {year}": "Artigos de {month} {day}, {year}",
    "Posts for {month} {year}": "Artigos de {month} {year}",
    "Previous post": "Artigo anterior",
    "Publication date": "Data de publicación",
    "RSS feed": "Sindicación RSS",
    "Read in English": "Ler en Inglés",
    "Read more": "Ler máis",
    "Skip to main content": "Saltar ó contido principal",
    "Source": "Fonte",
    "Subcategories:": "Subcategorías:",
    "Tags and Categories": "Etiquetas e categorías",
    "Tags": "Etiquetas",
    "Uncategorized": "Sen categoría",
    "Updates": "Actualizacións",
    "Write your page here.": "Escribe a túa páxina aquí.",
    "Write your post here.": "Escribe o teu artigo aquí.",
    "old posts, page %d": "Artigos vellos, páxina %d",
    "page %d": "páxina %d",
}
