/*
	jQuery Colorbox language configuration
	language: Slovenian (si)
	translated by: Boštjan Pišler (pisler.si)
*/
jQuery.extend(jQuery.colorbox.settings, {
	current: "Slika {current} od {total}",
	previous: "Prejšnja",
	next: "Naslednja",
	close: "Zapri",
	xhrError: "Vsebine ni bilo mogoče naložiti.",
	imgError: "Slike ni bilo mogoče naložiti.",
	slideshowStart: "Zaženi prezentacijo",
	slideshowStop: "Zaustavi prezentacijo"
});