# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "",
    "(active)": "",
    "Also available in:": "Διαθέσιμο και στα:",
    "Archive": "Αρχείο",
    "Authors": "",
    "Categories": "Κατηγορίες",
    "Comments": "Σχόλια",
    "LANGUAGE": "Ελληνικά",
    "Languages:": "Γλώσσες",
    "More posts about %s": "Περισσότερες αναρτήσεις για %s",
    "Newer posts": "Νεότερες αναρτήσεις",
    "Next post": "Επόμενη ανάρτηση",
    "No posts found.": "Δε βρέθηκαν αναρτήσεις",
    "Nothing found.": "Δε βρέθηκε περιεχόμενο",
    "Older posts": "Παλαιότερες αναρτήσεις",
    "Original site": "Ιστοσελίδα αρχικής ανάρτησης",
    "Posted:": "Αναρτήθηκε:",
    "Posts about %s": "Αναρτήσεις για %s",
    "Posts by %s": "",
    "Posts for year %s": "Αναρτήσεις για το έτος %s",
    "Posts for {month} {day}, {year}": "Αναρτήσεις στις {day} {month}, {year}",
    "Posts for {month} {year}": "Αναρτήσεις για τον {month} του {year}",
    "Previous post": "Προηγούμενη ανάρτηση",
    "Publication date": "Ημερομηνία δημοσίευσης",
    "RSS feed": "",
    "Read in English": "Διαβάστε στα Ελληνικά",
    "Read more": "Διαβάστε περισσότερα",
    "Skip to main content": "",
    "Source": "Πηγαίος κώδικας",
    "Subcategories:": "",
    "Tags and Categories": "Ετικέτες και κατηγορίες",
    "Tags": "Ετικέτες",
    "Uncategorized": "",
    "Updates": "",
    "Write your page here.": "",
    "Write your post here.": "",
    "old posts, page %d": "σελίδα παλαιότερων αναρτήσεων %d",
    "page %d": "σελίδα %d",
}
