# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "zostáva %d minút na čítanie",
    "(active)": "(aktívne)",
    "Also available in:": "Tiež dostupné v:",
    "Archive": "Archív",
    "Authors": "",
    "Categories": "Kategórie",
    "Comments": "Komentáre",
    "LANGUAGE": "Slovenčina",
    "Languages:": "Jazyky:",
    "More posts about %s": "Viac príspevkov o %s",
    "Newer posts": "Novšie príspevky",
    "Next post": "Nasledujúci príspevok",
    "No posts found.": "Žiadne príspevky nenájdené",
    "Nothing found.": "Nič nenájdené.",
    "Older posts": "Staršie príspevky",
    "Original site": "Pôvodná stránka",
    "Posted:": "Zverejnené:",
    "Posts about %s": "Príspevky o %s",
    "Posts by %s": "",
    "Posts for year %s": "Príspevky z roku %s",
    "Posts for {month} {day}, {year}": "Príspevky zo dňa {day}. {month} {year}",
    "Posts for {month} {year}": "Príspevky za mesiac {month} z roku {year}",
    "Previous post": "Predchádzajúci príspevok",
    "Publication date": "Dátum zverejnenia",
    "RSS feed": "RSS kanál",
    "Read in English": "Čítať v slovenčine",
    "Read more": "Čítať ďalej",
    "Skip to main content": "Skočiť na hlavný obsah",
    "Source": "Zdroj",
    "Subcategories:": "Podkategórie:",
    "Tags and Categories": "Štítky a kategórie",
    "Tags": "Štítky",
    "Uncategorized": "",
    "Updates": "",
    "Write your page here.": "Tu napíšte svoju stránku.",
    "Write your post here.": "Tu napíšte svoj príspevok.",
    "old posts, page %d": "staré príspevky, strana %d",
    "page %d": "stránka %d",
}
