/*
	jQuery Colorbox language configuration
	language: Croatian (hr)
	translated by: Mladen Bicanic (base.hr)
*/
jQuery.extend(jQuery.colorbox.settings, {
	current: "Slika {current} od {total}",
	previous: "Prethodna",
	next: "Sljedeća",
	close: "Zatvori",
	xhrError: "Neuspješno učitavanje sadržaja.",
	imgError: "Neuspješno učitavanje slike.",
	slideshowStart: "Pokreni slideshow",
	slideshowStop: "Zaustavi slideshow"
});