# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "%d min. tilbage at læse",
    "(active)": "",
    "Also available in:": "Fås også i:",
    "Archive": "Arkiv",
    "Authors": "",
    "Categories": "Kategorier",
    "Comments": "Kommentarer",
    "LANGUAGE": "Dansk",
    "Languages:": "Sprog:",
    "More posts about %s": "Yderligere indlæg om %s",
    "Newer posts": "Nyere indlæg",
    "Next post": "Næste indlæg",
    "No posts found.": "Søgningen gav ingen resultater.",
    "Nothing found.": "Søgningen gav ingen resultater.",
    "Older posts": "Ældre indlæg",
    "Original site": "Oprindeligt hjemmeside",
    "Posted:": "Opslået:",
    "Posts about %s": "Indlæg om %s",
    "Posts by %s": "",
    "Posts for year %s": "Indlæg for %s",
    "Posts for {month} {day}, {year}": "Indlæs for {month} {day}, {year}",
    "Posts for {month} {year}": "Indlæg for {month} {year}",
    "Previous post": "Tidligere indlæg",
    "Publication date": "Udgivelsesdato",
    "RSS feed": "RSS-nyhedskilde",
    "Read in English": "Læs på dansk",
    "Read more": "Læs mere",
    "Skip to main content": "Hop direkte til hovedindhold",
    "Source": "Kilde",
    "Subcategories:": "",
    "Tags and Categories": "Nøgleord og kategorier",
    "Tags": "Nøgleord",
    "Uncategorized": "",
    "Updates": "",
    "Write your page here.": "",
    "Write your post here.": "",
    "old posts, page %d": "gamle indlæg, side %d",
    "page %d": "side %d",
}
