#container {
    width: 960px;
    margin: 0 auto;
}

#contentcolumn {
    max-width: 760px;
}
#q {
    width: 150px;
}

img {
    max-width: 90%;
}

.postbox {
    border-bottom: 2px solid darkgrey;
    margin-bottom: 12px;
}

.titlebox {
    text-align: right;
}

#addthisbox {margin-bottom: 12px;}

td.label {
    /* Issue #290 */
    background-color: inherit;
}

.footnote-reference {
    /* Issue 290 */
    vertical-align: super;
    font-size: xx-small;
}


.caption {
    /* Issue 292 */
    text-align: center;
    padding-top: 1em;
}

div.figure > img,
div.figure > a > img {
    /* Issue 292 */
    display: block;
    margin-left: auto;
    margin-right: auto;
}

blockquote p, blockquote {
  font-size: 17.5px;
  font-weight: 300;
  line-height: 1.25;
}

ul.bricks > li {
    display: inline;
    background-color: lightblue;
    padding: 8px;
    border-radius: 5px;
    line-height: 3;
    white-space:nowrap;
    margin: 3px;
}

.at300b, .stMainServices, .stButton, .stButton_gradient {
    box-sizing: content-box;
}

pre, pre code {
    white-space: pre;
    word-wrap: normal;
    overflow: auto;
}

article.post-micro {
    font-family: Georgia, 'Times New Roman', Times, serif;
    font-size: 1.5em;
}

.image-block {
    display: inline-block;
}

.flowr_row {
    width: 100%;
}

.tags {
    padding-left: 0;
    margin-left: -5px;
    list-style: none;
    text-align: center;

}

.tags > li {
    display: inline-block;
    min-width: 10px;
    padding: 3px 7px;
    font-size: 12px;
    font-weight: bold;
    line-height: 1;
    color: #fff;
    text-align: center;
    white-space: nowrap;
    vertical-align: baseline;
    background-color: #999;
    border-radius: 10px;
}

.tags > li a {
    color: #fff;
}

.metadata p:before,
.postlist .listdate:after {
    content: " — ";
}

.metadata p:first-of-type:before {
    content: "";
}

.metadata p {
    display: inline;
}

.posttranslations h3 {
    display: inline;
    font-size: 1em;
    font-weight: bold;
}

.posttranslations h3:last-child {
    display: none;
}

.entry-content {
    margin-top: 1em;
}

.navbar-brand {
    padding: 0 15px;
}

.navbar-brand #blog-title {
    padding: 15px 0;
    display: inline-block;
}

.navbar-brand #logo {
    max-width: 100%;
}

.navbar-brand>img {
    display: inline;
}

.row {
    margin: 0;
}

/* for alignment with Bootstrap's .entry-content styling */
.entry-summary {
    margin-top: 1em;
}

/* Custom page footer */
#footer {
    padding-top: 19px;
    color: #777;
    border-top: 1px solid #e5e5e5;
}

/* hat tip bootstrap/html5 boilerplate */
@media print {
    *, *:before, *:after {
        font-family: Garamond, Junicode, serif;
    }

    body {
        font-size: 12pt;
    }

    article .entry-title a[href]:after,
    article .metadata a[href]:after,
    article .tags a[href]:after {
        content: "";
    }

    article .metadata .sourceline {
        display: none;
    }

    article .metadata .linkline a[href]:after {
        content: " (" attr(href) ")";
    }

    .navbar {
        display: none;
    }
}
