# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "",
    "(active)": "",
    "Also available in:": "Saadaval ka:",
    "Archive": "Arhiiv",
    "Authors": "",
    "Categories": "Kategooriad",
    "Comments": "",
    "LANGUAGE": "Eesti",
    "Languages:": "",
    "More posts about %s": "Veel postitusi %s kohta",
    "Newer posts": "Uued postitused",
    "Next post": "Järgmine postitus",
    "No posts found.": "",
    "Nothing found.": "",
    "Older posts": "Vanemad postitused",
    "Original site": "Algallikas",
    "Posted:": "Postitatud:",
    "Posts about %s": "Postitused %s kohta",
    "Posts by %s": "",
    "Posts for year %s": "Postitused aastast %s",
    "Posts for {month} {day}, {year}": "",
    "Posts for {month} {year}": "Postitused {year} aasta kuust {month} ",
    "Previous post": "Eelmine postitus",
    "Publication date": "",
    "RSS feed": "",
    "Read in English": "Loe eesti keeles",
    "Read more": "Loe veel",
    "Skip to main content": "",
    "Source": "Lähtekood",
    "Subcategories:": "",
    "Tags and Categories": "Sildid ja kategooriad",
    "Tags": "Märksõnad",
    "Uncategorized": "",
    "Updates": "",
    "Write your page here.": "",
    "Write your post here.": "",
    "old posts, page %d": "vanade postituste, leht %d",
    "page %d": "leht %d",
}
