/*
	jQuery Colorbox language configuration
	language: Korean (kr)
	translated by: lunareffect
*/
jQuery.extend(jQuery.colorbox.settings, {
	current: "총 {total} 중 {current}",
	previous: "이전",
	next: "다음",
	close: "닫기",
	xhrError: "컨텐츠를 불러오는 데 실패했습니다.",
	imgError: "이미지를 불러오는 데 실패했습니다.",
	slideshowStart: "슬라이드쇼 시작",
	slideshowStop: "슬라이드쇼 중지"
});
