/*
  jQuery Colorbox language configuration
	language: Galician (gl)
	translated by: donatorouco
*/
jQuery.extend(jQuery.colorbox.settings, {
	current: "Imaxe {current} de {total}",
	previous: "Anterior",
	next: "Seguinte",
	close: "Pechar",
	xhrError: "Erro na carga do contido.",
	imgError: "Erro na carga da imaxe."
});
