/*
	jQuery Colorbox language configuration
	language: Italian (it)
	translated by: maur8ino
*/
jQuery.extend(jQuery.colorbox.settings, {
	current: "Immagine {current} di {total}",
	previous: "Precedente",
	next: "Successiva",
	close: "Chiudi",
	xhrError: "Errore nel caricamento del contenuto.",
	imgError: "Errore nel caricamento dell'immagine.",
	slideshowStart: "Inizia la presentazione",
	slideshowStop: "Termina la presentazione"
});
